%This program gets a tide prediction of travel time in milliseconds
%averaged along a line between two points.  Used for direct comparison
%to the tomography data, or removing the barotropic signals from travel
%time timeseris.
%Edit the top of the file to give start and end locations of path, and
%to give the year and yeardays for the prediction.
clear
hold off
clf

%The HOME Southern array Positions are:
HH=[  -161.124  16.442  900  5583    56  5498  5714;
-162.638  18.416  900  5517    61  5302  5555;
-160.414  18.312  900  5238    43  5158  5312;
-158.652  17.022  900  5223    51  5156  5370];

%The HOME Northern array Positions are:
if 1==2,
HH=[    -163.732  28.780  900  5142   130  4905  5531;
        -161.944  26.919  900  5178    75  4926  5242;
        -164.311  26.860  900  4745    77  4668  4996;
        -166.342  27.965  900  4836    92  4723  5057];
end

%The RTE87 array Positions are:
if 1==2,
HH=[-157.140626 40.444618  675;
    -157.111321 31.473091  909;
    -165.929733 40.467537  856];
end

mooring1=4;
mooring2=3;

YEAR=2001;
yearday_start=273;
yearday_end = 475;

[RR,latw,lonw]=dist([HH(mooring1,2) HH(mooring2,2)],[HH(mooring1,1),HH(mooring2,1)],50);
%latw and lonw are the positions at which the tide model needs to be evaluated.

%load in all the tide data and the model bathymetry - 79MB
load haw/allconsts.mat
% now have h, elr, eli, ur, ui, vr, vi defined for all 8 constituents.
 
%load in the lat and lon values for h, el, u, v
load haw/latlons.mat
%now have lat,lon; latu, lonu; latv; lonv

AG=[];
%
%h and lat,lon; latu,lonu; latv;lonv are loaded in the initial program predict.m
%
% tm2 is M2 tidal period in hours
tm2=12.4206;
ts2=12.;
tn2=12.6583;
tk2=11.9672;
to1=25.8193;
tk1=23.9345;
tp1=24.0659;
tq1=26.8724;
omegat=48.*pi./[tm2 ts2 tn2 tk2 to1 tk1 tp1 tq1];
% omegat has the tidal frequencies in rad/yearday.
 
% the order of frequencies is M2, S2, N2, K2, O1, K1, P1, Q1
% ui,ur,vi,vr,eli,elr are all loaded in predict.m
Aall=[];
Gall=[];
for iop=1:8,

eval(['ui=ui' num2str(iop) ';'])
eval(['ur=ur' num2str(iop) ';'])
eval(['vi=vi' num2str(iop) ';'])
eval(['vr=vr' num2str(iop) ';'])
eval(['eli=eli' num2str(iop) ';'])
eval(['elr=elr' num2str(iop) ';'])

% WE WILL ONLY INTERPOLATE USING DATA WITHIN 2 GRID POINTS
% note there is potentially a problem near land because land has
% a value of zero for the tide - the interpolation will work using
% this value of zero.  Therefore, results within a grid point of
% land may be suspect.
%
DEL_LON=2*(lonu(1,2)-lonu(1,1));
DEL_LAT=2*(latu(2,1)-latu(1,1));
I1=((min(lonw)-DEL_LON) < lonu(1,:) & lonu(1,:) < (max(lonw)+DEL_LON)) ;
I2=((min(latw)-DEL_LAT) < latu(:,1) & latu(:,1) < (max(latw)+DEL_LAT));
if sum(I1)>=3 & sum(I2)>=3,
 
lon_tmp=lonu(I2,I1);
lat_tmp=latu(I2,I1);
ur_tmp=ur(I2,I1);
ui_tmp=ui(I2,I1);
UR=interp2(lon_tmp,lat_tmp,ur_tmp,lonw,latw,'*cubic');
UI=interp2(lon_tmp,lat_tmp,ui_tmp,lonw,latw,'*cubic');
Au14=sqrt(UR.^2 + UI.^2);
Gu14=atan2(-UI,UR)*(180/pi);
 
I1=((min(lonw)-DEL_LON) < lonv(1,:) & lonv(1,:) < (max(lonw)+DEL_LON)) ;
I2=((min(latw)-DEL_LON) < latv(:,1) & latv(:,1) < (max(latw)+DEL_LON));
lon_tmp=lonv(I2,I1);
lat_tmp=latv(I2,I1);
vr_tmp=vr(I2,I1);
vi_tmp=vi(I2,I1);
VR=interp2(lon_tmp,lat_tmp,vr_tmp,lonw,latw,'*cubic');
VI=interp2(lon_tmp,lat_tmp,vi_tmp,lonw,latw,'*cubic');
Av14=sqrt(VR.^2 + VI.^2);
Gv14=atan2(-VI,VR)*(180/pi);

I1=((min(lonw)-DEL_LON) < lon(1,:) & lon(1,:) < (max(lonw)+DEL_LON)) ;
I2=((min(latw)-DEL_LAT) < lat(:,1) & lat(:,1) < (max(latw)+DEL_LAT));
lon_tmp=lon(I2,I1);
lat_tmp=lat(I2,I1);
h_tmp=h(I2,I1);
elr_tmp=elr(I2,I1);
eli_tmp=eli(I2,I1);
h14=interp2(lon_tmp,lat_tmp,h_tmp,lonw,latw,'*cubic');
Ar=interp2(lon_tmp,lat_tmp,elr_tmp,lonw,latw,'*cubic');
Ai=interp2(lon_tmp,lat_tmp,eli_tmp,lonw,latw,'*cubic');

 
A=100*sqrt(Ar.^2 + Ai.^2);
% minus sign below because Gary's sign convention for phase is opposite
% to the rest of the world's.
G=(180/pi)*atan2(-Ai,Ar);
end

% the values of Au14 and Av14 are actually transports in m^2/s.  Divide by
% the model depth and multiply by 100 to convert to current in (cm/s)
Au14=100*Au14./h14;
Av14=100*Av14./h14;
Gu14( Gu14 > 360)=Gu14( Gu14 > 360)-360;
Gv14( Gv14 > 360)=Gv14( Gv14 > 360)-360;
Gu14( Gu14 <   0)=Gu14( Gu14 < 0  )+360;
Gv14( Gv14 <   0)=Gv14( Gv14 < 0  )+360;
G( G > 360)=G( G > 360)-360;
G( G <   0)=G( G < 0  )+360;


NNN=length(latw);
AA=[];
PP=[];
RR=[];
for i=1:(NNN-1);
   loc1=[latw(i) lonw(i)];
   loc2=[latw(i+1) lonw(i+1)];
   AZ=getazimuth(loc1,loc2);
   az=AZ*pi/180;
   r=range([latw(1) lonw(1)],loc1);
   RR=[RR r];
% az is in degrees with respect to north
   a1p=Au14(i)*sin(az);
   a2p=Av14(i)*cos(az);
   p1=Gu14(i)*pi/180;
   p2=Gv14(i)*pi/180;
%
   Q1=a1p*cos(p1)+a2p*cos(p2);
   Q2=a1p*sin(p1)+a2p*sin(p2);
%
   A=sqrt(Q1^2 + Q2^2);
   p=atan2(Q2,Q1)*180/pi;
   if p < 0,
     p=p+360;
   end
   AA=[AA A];
   PP=[PP p];
end
 
%  now have amplitude and phase in the azimuthal direction for
%  [latw lonw] at indices 1:(NNN-1)
clear j
tide=AA.*exp(j*PP*pi/180);
tide=mean(tide);
AMP=abs(tide);
PHA=angle(tide)*180/pi;
if PHA < 0,
PHA=PHA+360;
end

Aall=[Aall AMP];
Gall=[Gall PHA];

end
%end of loop over iop=1:8
%
% Now we have the range average amplitude and phase in Aall and Gall.
% Calculate the timeseries!

yd=yearday_start:(1/24):yearday_end;
yd=yd';
 
YD1=[yearday_start*0.98 yearday_end*1.02];
 
% begin with definition of some constants...
oneday=[335.62  0  322.55  1.97   334.63  0.99  -0.99  321.57];
% omegat has the tidal frequencies in rad/yearday.

load lun_node.dat
load v0u.dat
% the ordering of the constants in the above two files is:
%         M2   S2   N2      K2     O1     K1     P1      Q1
 
years=lun_node(:,1);
I=YEAR==years;
vou=v0u(I,2:9);
lunnod=lun_node(I,2:9);
 
JJ=1:8;
vou=(pi/180)*vou;
oneday=(pi/180)*oneday;
q=ones(size(yd));
 
% Calculation of currents - in cm/s.
G=(pi/180)*Gall;
A=Aall.*lunnod;
d=ones(size(yd))*A(JJ).*cos(yd*omegat(JJ) - q*oneday(JJ) + q*vou(JJ) - q*G );
ts1=sum(d')';

% convert ts1, which is in cm/s, to milliseconds.
% RR is in km - convert to m.
RR=RR(length(RR))*1000;
fac=-RR/(1480^2);
ts1=ts1*fac/100.;

%convert to milliseconds.
%ts1=ts1*1000;

plot(yd,ts1)

BB=[yd ts1];

h=fopen('linepred.out','w');
fprintf(h,'%12.5f %13.8f\n',BB');

disp('Variables yd and ts1 (in milliseconds) have been saved to "linepred.out"') 
